""" JSR-170 Item interface, Level1

$Id: item.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface
from zope.interface import Attribute

class IItem(Interface):
    """ IItem is an abstract base for INode and IProperty.
    """
    def getPath():
        """ Return the absolute path to this item.

        o If the path includes items that are same name sibling nodes or
         multi-value properties then those elements in the path will include
         the appropriate square bracket index notation (for example,
         '/a/b[3]/c').

        o Raise RepositoryException if an error occurs.
        """

    def getName():
        """ Return the name of this item.

        o The name is the last item in the path, minus any square-bracket
          index that may exist. If this item is the root node of the
          workspace (i.e., if 'getDepth() == 0'), returns an empty string.

        o Raise RepositoryException if an error occurs.
        """

    def getAncestor(depth):
        """ Return the ancestor of the specified depth below the root.

        o 'depth' is an integer, defining the requested number of levels.

        o An ancestor of depth x is the Item that is x levels down along the
          path from the root node to this Item. 

          - depth = 0 returns the root node. 

          - depth = 1 returns the child of the root node along the path
            to this Item.

          - depth = 2 returns the grandchild of the root node along the
            path to this Item.

          - And so on to depth = n, where n is the depth of this Item,
            which returns this Item itself.

        o Raise ItemNotFoundException will be thrown if depth < 0 or depth > n
          where n is the is the depth of this item along the path returned by
          'getPath()'.

        o Raise AccessDeniedException if the current session does not have
          sufficient access permissions to retrieve the specified node.

        o Raise RepositoryException if another error occurs.
        """

    def getParent():
        """ Return the parent of this Item.

        o Raise ItemNotFoundException if there is no parent node. This only
          happens if this item is the root node of a workspace.

        o Raise AccessDeniedException if the current session does not have
          sufficient access permissions to retrieve the parent of this item.

        o Raise RepositoryException is thrown if another error occurs.
        """

    def getDepth():
        """ Return the depth below the root node of this Item, counting this
            Item itself:

            - The root node returns 0.

            - A property or child node of the root node returns 1.

            - A property or child node of a child node of the root returns 2.

            - And so on to this Item.

        o Raise RepositoryException is thrown if an error occurs.
        """

    def getSession():
        """ Return the Session through which this Item was acquired.

        o Raise RepositoryException if an error occurs.
        """

    def isNode():
        """ Return True if this Item is a Node; returns False if this Item
            is a Property.
        """

    def isSame(otherItem):
        """ Return True if this Item object represents the same actual
            repository item as the object 'otherItem'.

        o This method does not compare the states of the two items.
          For example, if two Item objects representing the same actual
          repository item have been retrieved through two different
          sessions and one has been modified, then this method will still
          return True when comparing these two objects.

        o Note that if two Item objects representing the same repository item
          are retrieved through the same session they will always reflect the
          same state, so comparing state is not an issue.

        o Raise RepositoryException if an error occurs.
        """

    # accept() is not defined, because its behavior relies on Java's
    # method overloading, to achieve an effect that Python would achieve
    # by duck-typing, and Zope3 by using an adapter.
