""" JSR-170 Node interface, Level 1

$Id: node.py 515 2005-07-11 18:00:47Z tseaver $
"""
from jsr170.level1.item import IItem

class INode(IItem):
    """ A node represents an introspectable element of content space.
    """
    def getNode(relPath):
        """ Return the node at 'relPath' relative to this node.

        o If 'relPath' contains a path element that refers to a node with
          same-name sibling nodes without explicitly including an index
          using the array-style notation ([x]), then the index [1] is assumed 

        o Within the scope of a single Session object, if a node has been
          acquired with getNode, any subsequent call of 'getNode' reacquiring
          the same node must return a Node object reflecting the same state
          as the earlier Node object.  Whether this object is actually the
          same Node instance, or simply one wrapping the same state, is up to
          the implementation.

        o Raise PathNotFoundException if no node exists at relPath.

        o Raise RepositoryException if another error occurs.
        """

    def getNodes(namePattern='*'):
        """ Return an iterator over all child nodes of this node whose names
            match 'namePattern'.

        o The pattern may be a full name or a partial name with one or more
          wildcard characters ("*"), or a disjunction (using the  "|" character
          to represent logical OR) of these. For example,
          'N.getNodes("jcr:* | myapp:report | my doc")' would return an
          iterator holding all child nodes of N that are either called
          "myapp:report", begin with the prefix "jcr:" or are called "my doc".

        o Note that leading and trailing whitespace around a "|" character is
          ignored, but whitespace within a disjunct forms part of the pattern
          to be matched.

        o The EBNF for namePattern is::

          namePattern ::= disjunct {'|' disjunct}

          disjunct ::= name [':' name]

          name ::= '*' | ['*'] fragment {'*' fragment}['*']

          fragment ::= char {char}

          char ::= nonspace | ' '

          nonspace ::= (* Any Unicode character except:
            '/', ':', '[', ']', '*', ''', '"', '|' or any whitespace character
          *)

          The pattern is matched against the names (not the paths) of
          the immediate child nodes of this node.

        o Results do not include properties of this node.

        o The same reacquisition semantics apply as with 'getNode'.

        o If this node has no child nodes, an empty iterator is returned.

        o Raise RepositoryException if an error occurs.
        """

    def getProperty(relPath):
        """ Return the property at 'relPath' relative to this node.

        o The same reacquisition semantics apply as with 'getNode'.

        o Raise PathNotFoundException if no property exists at 'relPath'.

        o Raise RepositoryException if another error occurs.
        """

    def getProperties(namePattern="*"):
        """ Return an iterator of all properties of this node whose names
            match 'namePattern'.

        o The pattern may be a full name or a partial name with one or more
          wildcard characters ("*"), or a disjunction (using the "|" character
          to represent logical OR) of these. For example,
          'N.getProperties("jcr:* | myapp:name | my doc")' would return an
          iterator holding all properties of N that are either called
          "myapp:name", begin with the prefix "jcr:" or are called "my doc".

        o Note that leading and trailing whitespace around a disjunct is
          ignored, but whitespace within a disjunct forms part of the pattern
          to be matched.

        o The EBNF for namePattern is::

          namePattern ::= disjunct {'|' disjunct}

          disjunct ::= name [':' name]

          name ::= '*' | ['*'] fragment {'*' fragment}['*']

          fragment ::= char {char}

          char ::= nonspace | ' '

          nonspace ::= (* Any Unicode character except:
           '/', ':', '[', ']', '*', ''', '"', '|' or any whitespace character
          *)

        o Results do not include child nodes of this node.

        o The pattern is matched against the names (not the paths) of the
          immediate child properties of this node.

        o If this node has no matching properties, then an empty iterator is
          returned.

        o The same reacquisition semantics apply as with 'getNode'.

        o Raise RepositoryException if an error occurs.
        """

    def getPrimaryItem():
        """ Return the primary child item (node or property) of this node,
            as spcified by its primary node type

        o The same reacquisition semantics apply as with 'getNode'.

        o Raise ItemNotFoundException if this node has no primary child item,
          either because none is declared in the node type or because a
          declared primary item is not present on this node instance,

        o Raise RepositoryException if another error occurs.
        """

    def getUUID():
        """ Return the UUID of this node as recorded in the node's
            'jcr:uuid' property.

        o This method only works on nodes of mixin node type
          'mix:referenceable'.

        o Raise UnsupportedRepositoryOperationException if the node is
          nonreferenceable.

        o To determine whether a node has a UUID, call
          'isNodeType("mix:referenceable")'.

        o Raise RepositoryException if another error occurs.
        """

    def getIndex():
        """ Return the index of this node within the ordered set of its
            same-name sibling nodes.

        o This index is the one used to address same-name siblings using the
          square-bracket notation, e.g., "/a[3]/b[4]".

        o Note that the index always starts at 1 (not 0), for compatibility
          with XPath. As a result, for nodes that do not have
          same-name-siblings, this method will always return 1.
        """

    def getReferences():
        """ Return all REFERENCE properties that refer to this node.

        o Some level 2 implementations may only return properties that have
          been saved (in a transactional setting this includes both those
          properties that have been saved but not yet committed, as well as
          properties that have been committed). Other level 2 implementations
          may additionally return properties that have been added within the
          current Session but are not yet saved.

        o In implementations that support versioning, this method does not
          return REFERENCE properties that are part of the frozen state of a
          version in version storage.

        o If this node has no references, an empty iterator is returned.

        o Raise RepositoryException if an error occurs.
        """

    def hasNode(relPath):
        """ Return True if a node exists at 'relPath', else False.
        """

    def hasNodes():
        """ Return True if this node has one or more child nodes, else False.
        """

    def hasProperty(relPath):
        """ Return True if a property exists at 'relPath', else False.
        """

    def hasProperties():
        """ Return True if this node has one or more properties, else False.
        """

    def getPrimaryNodeType():
        """ Return the primary INodeType assigned to this node.

        o Which NodeType is returned when this method is called on the
          root node of a workspace is up to the implementation, though the
          returned type must, of course, be consistent with the child nodes
          and properties of the root node.

        o Raise RepositoryException if an error occurs.
        """

    def getMixinNodeTypes():
        """ Return a sequence of INodeType objects representing the mixin
            node types assigned to this node.

        o The returned sequence includes only those mixin types explicitly
          assigned to this node, and therefore listed in the property
          'jcr:mixinTypes'. It does not include mixin types inherited through
          the addition of supertypes to the primary type hierarchy.

        o Raise RepositoryException if an error occurs.
        """

    def isNodeType(nodeTypeName):
        """ Return True if this node is of the specified primary node type
            or mixin type, or a subtype thereof;  else False.

        o 'nodeTypeName' is a string identifying the queried node type.

        o Raise RepositoryException if an error occurs.
        """

    def getDefinition():
        """ Return the INodeDefinition that applies to this node.

        o In some cases there may appear to be more than one definition
          that could apply to this node. However, it is assumed that upon
          creation of this node, a single particular definition was used and
          it is that definition that this method returns.

        o How this governing definition is selected upon node creation
          from among others which may have been applicable is an
          implementation issue and is not covered by this specification.

        o The NodeDefinition returned when this method is called on the root
          node of a workspace is also up to the implementation.

        o Raise RepositoryException if an error occurs.
        """
