""" JSR-170 Repository interface. Level 1

$Id: repository.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface
from zope.interface import Attribute

class IRepository(Interface):
    """ A repository represents the gateway to a collection of workspaces.

    o See javax.jcr.Repository for the Java version of this interface.
    """

    def login(credentials=None, workspaceName=None):
        """ Authenticate the user using the supplied 'credentials'.
     
        o Return a valid session (ISession) which allows the user to access
          the repository.
     
        o 'credentials' is the credentials of the user.  It must implement
          jsr170.level1.ICredentials.

          If 'credentials' is None, it is assumed that authentication is
          handled by a mechanism external to the repository itself (for
          example, through the JAAS framework) and that the repository
          implementation exists within a context (for example, an application
          server) that allows it to handle authorization of the request for
          access to the specified workspace.
     
          Raise LoginException if the login fails
     
        o 'workspaceName' is the string name of a workspace in the repository

          The format of the string 'workspaceName' depends upon the
          implementation.

          If 'workspaceName' is recognized as the name of an existing
          workspace in the repository and authorization to access that
          workspace is granted, then a new 'ISession' object is returned.

          If 'workspaceName' is None, a default workspace is automatically
          selected by the repository implementation. This may, for example,
          be the "home workspace" of the user whose credentials were passed,
          though this is entirely up to the configuration and implementation
          of the repository. Alternatively, it may be a "null workspace" that
          serves only to provide the method 'getAccessibleWorkspaceNames',
          allowing the client to select from among available "real" workspaces.
     
          Raise NoSuchWorkspaceException if the specified 'workspaceName'
          is not recognized.
 
        o Raise RepositoryException if another error occurs.
        """

    def getDescriptorKeys():
        """ Return a sequence of all descriptor keys available for this
            implementation.

        o This set must contain at least the built-in keys defined by
          the string constants in this interface.
           
        o Used in conjunction with 'getDescriptor(String name)'
          to query information about this repository implementation.
        """

    def getDescriptor(key):
        """ Return the descriptor for the specified key.
 
        o 'key' is a string corresponding to a descriptor for this repository
          implementation.
        
        o Used to query information about this repository implementation.
        
        o The set of available keys can be found by calling
          'getDescriptorKeys'.

        o If the specifed key is not found, return None.
        """

    SPEC_VERSION_DESC = Attribute('SPEC_VERSION_DESC',
        """jcr.specification.version

        o The descriptor key for the version of the specification that
        this repository implements.
        
        o required

        o type is unicode

        o value is u"1.0",
        """
        )

    SPEC_NAME_DESC = Attribute('SPEC_NAME_DESC',
        """jcr.specification.name

        o The descriptor key for the name of the specification that this
          repository implements

        o required

        o type is unicode

        o value is u"Content Repository for Java Technology API"
        """
        )

    REP_VENDOR_DESC = Attribute('REP_VENDOR_DESC',
        """jcr.repository.vendor

        o The descriptor key for the name of the repository vendor

        o required

        o type is unicode
        """
        )

    REP_VENDOR_URL_DESC = Attribute('REP_VENDOR_URL_DESC',
        """jcr.repository.vendor.url

        o The descriptor key for the URL of the repository vendor
        
        o required
        
        o type is unicode
        """
        )

    REP_NAME_DESC = Attribute('REP_NAME_DESC',
        """jcr.repository.name

        o The descriptor key for the name of this repository implementation
        
        o required

        o type is unicode
        """
        )

    REP_VERSION_DESC = Attribute('REP_VERSION_DESC',
        """jcr.repository.version

        o The descriptor key for the version of this repository implementation.

        o required
        
        o type is unicode
        """
        )

    LEVEL_1_SUPPORTED = Attribute('LEVEL_1_SUPPORTED',
        """level.1.supported

        o The presence of this key indicates that this implementation supports
          all level 1 features.  This key will always be present.

        o required

        o type is unicode

        o value is u"True"
        """
        )

    LEVEL_2_SUPPORTED = Attribute('LEVEL_2_SUPPORTED',
        """level.2.supported

        The presence of this key indicates that this implementation
        supports all level 2 features.
        
        o optional

        o value is u"True", if present.
        """
        )

    OPTION_TRANSACTIONS_SUPPORTED = Attribute('OPTION_TRANSACTIONS_SUPPORTED',
        """option.transactions.supported

        o The presence of this key indicates that this implementation supports
          transactions.
        """
        )

    OPTION_VERSIONING_SUPPORTED = Attribute('OPTION_VERSIONING_SUPPORTED',
        """option.versioning.supported

        The presence of this key indicates that this implementation supports
        versioning
        """
        )

    OPTION_OBSERVATION_SUPPORTED = Attribute('OPTION_OBSERVATION_SUPPORTED',
        """option.observation.supported

        The presence of this key indicates that this implementation supports
        observation (a Javaism for event notification).
        """
        )

    OPTION_LOCKING_SUPPORTED = Attribute('OPTION_LOCKING_SUPPORTED',
        """option.locking.supported

        The presence of this key indicates that this implementation supports
        locking.
        """
        )

    OPTION_QUERY_SQL_SUPPORTED = Attribute('OPTION_QUERY_SQL_SUPPORTED',
        """option.query.sql.supported

        The presence of this key indicates that this implementation
        supports the SQL query language.
        """
        )

    QUERY_XPATH_POS_INDEX = Attribute('QUERY_XPATH_POS_INDEX',
        """query.xpath.pos.index

        The presence of this key indicates that the index position notation
        for same-name siblings is supported within XPath queries.
        """
        )

    QUERY_XPATH_DOC_ORDER = Attribute('QUERY_XPATH_DOC_ORDER',
        """query.xpath.doc.order

        The presence of this key indicates that XPath queries return
        results in document order.
        """
        )
