""" JSR-170 Session interface. Level 1

$Id: session.py 515 2005-07-11 18:00:47Z tseaver $
"""
from zope.interface import Interface
from zope.interface import Attribute

class ISession(Interface):
    """ Provide read and (in level 2) write access to the content of a
        particular workspace in the repository.

    o The ISession object is returned by 'IRepository.login'.  It
      encapsulates both the authorization settings of a particular user
      (as specified by the passed ICredentials) and a binding to the
      workspace specified by the 'workspaceName' passed on 'login'.

    o Each ISession object is associated one-to-one with an IWorkspace
      object.  The IWorkspace object represents a "view" of an actual
      repository workspace entity as seen through the authorization settings
      of its associated ISession.
    """
    def getRepository():
        """ Return the IRepository object through which this session was
            acquired.
        """

    def getUserID():
        """ Return the user ID that was used to acquire this session.

        o This method is free to return an "anonymous user id" or None
          if the 'ICredentials' used to acquire this session happens not
          to have provided a real user ID.
        """

    def getAttributeNames():
        """ Return a sequence of attribute names of the session's user.
        """

    def getAttribute(name):
        """ Return the value of the named attribute of the session's user.

        o 'name' is a string identifying the requested attribute.

        o Return None if no attribute of the given name exists.
        """

    def getWorkspace():
        """ Return the IWorkspace attached to this session.
        """

    def getRootNode():
        """ Return the root node of the workspace.

        o The root 'INode', "/", is the main access point to the content of
          the workspace.

        o Raise RepositoryException if an error occurs.
        """

    def getItem(absPath):
        """ Return the 'IItem' at the specified absolute path in the workspace.

        o Raise PathNotFoundException if no such node exists.

        o Raise RepositoryException if another error occurs.
        """

    def itemExists(absPath):
        """ Return True if an item exists at 'absPath', otherwise False.

        o Raise RepositoryException if an error occurs.
        """

    def getNodeByUUID(uuid):
        """ Return the 'INode' specified by the given UUID.

        o Only applies to nodes that expose a UUID, in other words thoses of
          mixin node type 'mix:referenceable'.

        o Raise ItemNotFoundException if no such node exists.

        o Raise RepositoryException if another error occurs.
        """

    def impersonate(credentials):
        """ Return a new session using the supplied credentials.

        o Allows the current user to "impersonate" another using incomplete
          credentials (perhaps including a user name but no password,
          for example), assuming that their original session gives them that
          permission.

        o The new 'ISession' is tied to a new 'IWorkspace' instance.
          In other words, 'IWorkspace' instances are not re-used. However,
          the 'IWorkspace' instance returned represents the same actual
          persistent workspace entity in the repository as is represented
          by the 'IWorkspace' object tied to this 'ISession'.

        o Raise LoginException if the current session does not have
          sufficient rights.
        """

    def logout():
        """ Release all resources associated with this 'Session'.

        o This method should be called when a 'Session' is no longer needed.
        """

    def isLive():
        """ Return True if this ISession object is usable by the client,
            else False.

        o A usable 'Session' is one that is neither logged-out, timed-out
          nor in any other way disconnected from the repository.
        """

    def setNamespacePrefix(prefix, uri):
        """ Within the scope of this session, rename a persistently registered
            namespace URI to the new prefix.

        o 'prefix' is a string used as a shortcut handle for an XML namespace.

        o 'uri' is a string identifying uniquely an XML namespace.

        o The renaming only affects operations done through this session.

        o To clear all renamings the client must acquire a new session.

        o Raise NamespaceException on an attempt to remap a prefix that is
          currently already mapped to some URI (either persistently in the
          repository 'NamespaceRegistry' or transiently within this 'Session')
          since this would make any content stored using the old URI
          unreadable.

        o Raise NamespaceException if an attempt is made to remap an
          existing namespace URI to a prefix beginning with the characters
          "'xml'" * (in any combination of case).

        o Raise NamespaceException if the specified URI is not among those
          registered in the NamespaceRegistry.

        o Raise RepositoryException if another error occurs.
        """

    def getNamespacePrefixes():
        """ Return a sequence of all prefixes currently set for this session.

        o This includes all those registered in the 'NamespaceRegistry' but
          *not over-ridden* by a 'Session.setNamespacePrefix', plus those
          currently set locally by 'Session.setNamespacePrefix'.

        o Raise RepositoryException if an error occurs.
        """

    def getNamespaceURI(prefix):
        """ Return the URI to which 'prefix' is currently mapped.

        o 'prefix' is a string used as a shortcut handle for an XML namespace.

        o Raise NamespaceException if the prefix is unknown.

        o Raise RepositoryException if another error occurs.
        """

    def getNamespacePrefix(uri):
        """ Return the prefix to which the given URI is mapped.

        o 'uri' is a string identifying uniquely an XML namespace.

        o Raise NamespaceException if the URI is unknown.

        o Raise RepositoryException if another error occurs.
        """

    def exportSystemView(absPath, contentHandler=None, outputStream=None,
                         skipBinary=False, noRecurse=False):
        """ Serialize the node (and if 'noRecurse' is False, the whole
            subtree) at 'absPath', in the 'system view' form.

        o 'absPath' The path of the root of the subtree to be serialized.
          This must be the path to a node, not a property.

        o 'contentHandler' the  'xml.sax.handler.ContentHandler' to which
          the SAX events representing the XML serialization of the subtree
          will be output.

        o 'outputStream' The 'OutputStream' to which the XML
          serialization of the subtree will be output.

        o 'skipBinary' A 'boolean' governing whether binary properties are to
          be serialized.

        o 'noRecurse' A 'boolean' governing whether the subtree at 'absPath'
          is to be recursed.

        o One of 'contentHandler' or 'outputStream' must be non-None, but
          not both.

          - If 'contentHandler' is not None, serializies the node as a series
            of SAX events by calling the methods of the supplied
            'xml.sax.handler.ContentHandler'.

          - Otherwise, serializes the node as an XML stream and outputs it
            onto 'outputStream'.

        o If 'skipBinary' is True then any properties of type
          'PropertyType.BINARY' will be serialized as if they are empty.
          That is, the existence of the property will be serialized, but
          its content will not appear in the serialized output (the
          '<sv:value>' element will have no content). Note that in the case of
          multi-value 'BINARY' properties, the number of values in the property
          will be reflected in the serialized output, though they will all be
          empty.

        o If 'skipBinary' is False then the actual value(s) of each 'BINARY'
          property is recorded using Base64 encoding.

        o If 'noRecurse' is True then only the node at 'absPath' and its
          properties, but not its child nodes, are serialized.

        o If 'noRecurse' is False then the entire subtree rooted at 'absPath'
          is serialized.

        o If the user lacks read access to some subsection of the specified
          tree, that section simply does not get serialized, since, from the
          user's point of view, it is not there.

        o The serialized output will reflect the state of the current
          workspace as modified by the state of this 'Session'. This means
          that pending changes (regardless of whether they are valid according
          to node type constraints) and the current session-mapping of
          namespaces are reflected in the output.

        o Raise PathNotFoundException if no node exists at 'absPath'.

        o Raise SAXException if an error occurs while feeding events to the
          'ContentHandler'.

        o Raise IOError if an error occurs while writing to 'outputStream'.

        o Raise RepositoryException if another error occurs.
        """

    def exportDocumentView(absPath, contentHandler=None, outputStream=None,
                         skipBinary=False, noRecurse=False):
        """ Serialize the node (and if 'noRecurse' is False, the whole
            subtree) at 'absPath', in the 'document view' form.

        o 'absPath' The path of the root of the subtree to be serialized.
          This must be the path to a node, not a property.

        o 'contentHandler' the  'xml.sax.handler.ContentHandler' to which the
          SAX events representing the XML serialization of the subtree will be
          output.

        o 'outputStream' The 'OutputStream' to which the XML
          serialization of the subtree will be output.

        o 'skipBinary' A 'boolean' governing whether binary properties are to
          be serialized.

        o 'noRecurse' A 'boolean' governing whether the subtree at 'absPath'
          is to be recursed.

        o One of 'contentHandler' or 'outputStream' must be non-None, but
          not both.

          - If 'contentHandler' is not None, serializies the node as a series
            of SAX events by calling the methods of the supplied
            'xml.sax.handler.ContentHandler'.

          - Otherwise, serializes the node as an XML stream and outputs it
            onto 'outputStream'.

        o If 'skipBinary' is True then any properties of type
          'PropertyType.BINARY' will be serialized as if they are empty.
          That is, the existence of the property will be serialized, but
          its content will not appear in the serialized output (the
          '<sv:value>' element will have no content). Note that in the case of
          multi-value 'BINARY' properties, the number of values in the property
          will be reflected in the serialized output, though they will all be
          empty.

        o If 'skipBinary' is False then the actual value(s) of each 'BINARY'
          property is recorded using Base64 encoding.

        o If 'noRecurse' is True then only the node at 'absPath' and its
          properties, but not its child nodes, are serialized.

        o If 'noRecurse' is False then the entire subtree rooted at 'absPath'
          is serialized.

        o If the user lacks read access to some subsection of the specified
          tree, that section simply does not get serialized, since, from the
          user's point of view, it is not there.

        o The serialized output will reflect the state of the current
          workspace as modified by the state of this 'Session'. This means
          that pending changes (regardless of whether they are valid according
          to node type constraints) and the current session-mapping of
          namespaces are reflected in the output.

        o Raise PathNotFoundException if no node exists at 'absPath'.

        o Raise SAXException if an error occurs while feeding events to the
          'ContentHandler'.

        o Raise IOError if an error occurs while writing to 'outputStream'.

        o Raise RepositoryException if another error occurs.
        """

    def checkPermission(absPath, actions):
        """ Determine whether this ISession has permission to perform
           the specified 'actions' at the specified 'absPath'.
           
        o Return quietly if the access request is permitted.
        
        o Raise a suitable AccessDeniedException otherwise.

        o The 'actions' parameter is a comma separated list of action strings.
          The following action strings are defined:

          'add_node' -- has permission to add a node at 'absPath'     

          'set_property' -- has permission to set (add or change) a property
           at 'absPath',

          'remove' -- has permission to remove an item at 'absPath'     
          
          'read' -- has permission to retrieve (and read the value of, in
          the case of a property) an item at 'path'
          
        o When more than one action is specified in the 'actions' parameter,
          only return quietly if this 'Session' has permission to perform
          *all* of the listed actions at the specified path.

        o The information returned through this method will only reflect
          access control policies and not other restrictions that may exist.
          For example, even though 'checkPermission' may indicate that a
          particular 'Session' may add a property at '/A/B/C', the node type
          of the node at '/A/B' may prevent the addition of a property
          called 'C'.
        """
