""" JSR-170 Item interface, Level 2

$Id: item.py 516 2005-07-11 18:08:28Z tseaver $
"""
from jsr170.level1 import IItem as IItemLevel1

class IItem(IItemLevel1):
    """ Extend IItemLevel1 to allow writes.
    """
    def save():
        """ Validate all pending changes currently recorded in this ISession
            that apply to this item or any of its descendants (that is, the
            subtree rooted at this item).
            
        o If validation of all pending changes succeeds, then clear this change
          information from the Session.
          
        o If the save occurs outside a transaction, persist the changes,
          thus making them visible to other Sessions.
          
        o If the save occurs within a transaction, do not persist the changes 
          until the transaction is committed.
          
        o If validation fails, then no pending changes are saved and they
          remain recorded on the Session. There is no best-effort or partial
          save.
         
        o The item in persistent storage to which a transient item is saved
          is determined by matching UUIDs and paths.
          
        o Raise AccessDeniedException will be thrown if any of the
          changes to be persisted would violate the access permissions of
          the Session.
          
        o Raise ReferentialIntegrityException if any of the changes to be
          persisted would cause the removal of a node that is currently the
          target of a REFERENCE property, provided that this Session has
          read access to that REFERENCE property.
          
        o If, on the other hand, this Session does not have read access to
          the REFERENCE property in question, then raise AccessDeniedException
          is thrown instead.
          
        o Raise ItemExistsException if any of the changes to be persisted
          would be prevented by the presence of an already existing item in
          the workspace.
          
        o Raise ConstraintViolationException if any of the changes to be
          persisted would violate a node type restriction. Additionally, a
          repository may use this exception to enforce implementation- or
          configuration-dependant restrictions.

        o Raise InvalidItemStateException any of the changes to be persisted
          conflicts with a change already persisted through another session
          and the implementation is such that this conflict can only be
          detected at save-time and therefore was not detected earlier,
          at change-time.
          
        o Raise VersionException if the save would result in a change to
          persistent storage that would violate the read-only status of a
          checked-in node.

        o Raise LockException if the save would result in a change to
          persistent storage that would violate a lock
          
        o Raise NoSuchNodeTypeException if the save would result in the
          addition of a node with an unrecognized node type.
          
        o Raise RepositoryException if another error occurs.
        """

    def refresh(keepChanges):
        """Revert the item to its state in persistent storage.
            
        o Outside a transaction this state is simply the current state of
          persistent storage.
          
        o Within a transaction, this state will reflect persistent storage
          as modified by changes that have been saved but not yet committed.
        
        o If 'keepChanges' is False, this method discards all pending
          changes currently recorded in this Session that apply to this
          Item or any of its descendants (that is, the subtree rooted at
          this Item) and returns these items to reflect the current saved
          state.
          
        o If 'keepChanges' is True then pending changes are not discarded but
          items (within the subtree rooted at this Item) that do not have
          changes pending have their state refreshed to reflect the current
          saved state, thus revealing changes made by other sessions.
          
        o In some implementations this may be a null operation.
        
        o Raise InvalidItemStateException if this Item object represents a
          workspace item that has been removed (either by this session or
          another).
          
        o Raise RepositoryException if another error occurs.
        """

    def isNew():
        """ Return True if this is a new item, meaning that it exists only
            in transient storage on the Session and has not yet been saved;
            else False.
            
        o Within a transaction, 'isNew' on an Item may return False
          (because the item has been saved) even if that Item is not in
          persistent storage (because the transaction has not yet been
          committed).
          
        o Note that in level 1 (that is, read-only) implementations,
          this method will always return False.
        """

    def isModified():
        """ Return True if this Item has been saved but has subsequently
            been modified through the current session and therefore the
            state of this item as recorded in the session differs from the
            state of this item as saved;  else False.
            
        o Within a transaction, 'isModified' on an Item may return False
          (because the Item has been saved since the modification) even if
          the modification in question is not in persistent storage (because
          the transaction has not yet been committed).
          
        o Note that in level 1 (that is, read-only) implementations, this
          method will always return False.
        """

    def remove():
        """ Remove this item (and its subtree).
        
        o To persist a removal, a save must be performed that includes
          the (former) parent of the removed item within its scope.
          
        o Raise ReferentialIntegrityException on save if this item or an
          item in its subtree is currently the target of a REFERENCE property
          located in this workspace but outside this item's subtree and the
          current Session has read access to that REFERENCE property.
          
        o Raise AccessDeniedException on save if this item or an item in
          its subtree is currently the target of a REFERENCE property located
          in this workspace but outside this item's subtree and the current
          Session does not have read access to that REFERENCE property.
          
        o Raise ConstraintViolationException either immediately (by this
          method), or on save, if removing this item would violate a node
          type or implementation-specific constraint.  Implementations may
          differ on when this validation is performed.
          
        o Raise VersionException either immediately (by this method), or on
          save, if the parent node of this item is versionable and checked-in
          or is non-versionable but its nearest versionable ancestor is
          checked-in.  Implementations may differ on when this validation
          is performed.
          
        o Raise LockException either immediately (by this method), or on
          save, if a lock prevents the removal of this item.  Implementations
          may differ on when this validation is performed.
          
        o Raise RepositoryException if another error occurs.
        """
