""" JSR-170 Node interface, Level 2

$Id: node.py 516 2005-07-11 18:08:28Z tseaver $
"""
from jsr170.level1 import INode as INodeLevel1

_MARKER = object()

class INode(INodeLevel1):
    """ Extend INodeLevel1 to allow writes.
    """
    def addNode(relPath, primaryNodeTypeName=None):
        """ Create a new node at 'relPath', and return the new node.
        
        o 'relPath' is a string path, identifying the location of the new
          node.

        o 'primaryNodeTypeName', if passed, is a string identifying the type
          of the node to create.

        o The new node will only be persisted on save if it meets the
          constraint criteria of the parent node's node type. In order to
          save a newly added node, save must be called either on the Session,
          or on the new node's parent or higher-order ancestor
          (grandparent, etc.).
          
        o Raise NoSuchNodeTypeException either immediately (by this method),
          or on save, if the 'primaryNodeTypeName' is not recognized.
          Implementations may differ on when this validation is performed.

        o Raise RepositoryException on an attempt to call 'save' only on the
          newly added node.
          
        o Raise RepositoryException if the 'relPath' provided has an index
          on its final element.  Strictly speaking, the parameter is actually
          a relative path to the parent node of the node to be added,
          appended with the name desired for the new node 
          
          - if the node is being added directly below this node then only
            the name need be specified
          
          - 'relPath' does not specify a position within the child node
            ordering (if such ordering is supported). If ordering is supported
            by the node type of the parent node then the new node is appended
            to the end of the child node list.
            
        o If 'primaryNodeTypeName' is None, the new node's primary node type
          will be determined (either immediately or on save depending on the
          implementation) by the child node definitions in the node types of
          its parent.

        o Raise ItemExistsException either immediately (by this method), or
          on save, if an item at the specified path already exists and
          same-name siblings are not allowed.  Implementations may differ on
          when this validation is performed.
          
        o Raise PathNotFoundException either immediately (by this method), or
          on save, if the specified path implies intermediary nodes that do
          not exist.  Implementations may differ on when this validation is
          performed.
          
        o Raise ConstraintViolationException either immediately (by this
          method), or on save, if adding the node would violate a node type
          or implementation-specific constraint or if an attempt is made to
          add a node as the child of a property.  Implementations may differ
          on when this validation is performed.
          
        o Raise VersionException either immediately (by this method), or on
          save, if the node to which the new child is being added is
          versionable and checked-in or is non-versionable but its nearest
          versionable ancestor is checked-in.  Implementations may differ on
          when this validation is performed.
          
        o Raise LockException either immediately (by this method), or on save,
          if a lock prevents the addition of the node.  Implementations may
          differ on when this validation is performed

        o Raise RepositoryException if another error occurs.
        """

    def setProperty(name, value=_MARKER, values=_MARKER):
        """ Set the specified property of this node to the given value(s).

        o One of 'value' or 'values', but not both, must be passed.

        o 'value', if passed, must be adatpatble to IValue.

        o 'values', if passed, must be a sequence of objects of the same type,
          each adaptable to IValue.

        o If the property does not yet exist, it is created.
        
        o The property type of the property will be that specified by the
          node type of this node.
          
        o If the property type of the supplied Value object is different
          from that required, then a best-effort conversion is attempted.
          If the conversion fails, raise ValueFormatException.
          
        o Raise RepositoryException if another error occurs.
        
        o If the node type of this node does not indicate a specific
          property type, then the property type of the supplied Value object
          is used and if the property already exists (has previously been
          set) it assumes both the new value and new property type.
          
        o If 'value' is passed, and the property is multi-valued, raise
          ValueFormatException.
          
        o If 'values' is passed, and the property is not multi-valued, raise
          ValueFormatException.

        o Raise ValueFormatException if 'values' is passed with items of
          different types.
          
        o If 'value' or 'values' is passed as None, then remove the
          property.  This operation is equivalent to calling remove on the
          Property object itself.  For example, 'N.setProperty("P", None)'
          would remove the property called "P" of the node in N.
          
        o To save the addition or removal of a property, a 'save' call
          must be performed that includes the parent of the property in
          its scope: that is, a save on either the session, this node, or
          an ancestor of this node.
          
        o To save a change to an existing property, a save call that
          includes that property in its scope is required. This means that
          in addition to the above-mentioned save options, a save on the
          changed property itself will also work.
          
        o Raise ConstraintViolationException either immediately (by this
          method), or on save, if the change would violate a node type or
          implementationspecific constraint. Implementations may differ on
          when this validation is performed.
          
        o Raise VersionException either immediately (by this method), or on
          save, if this node is versionable and checked-in or is
          non-versionable but its nearest versionable ancestor is checked-in.
          Implementations may differ on when this validation is performed.
          
        o Raise LockException either immediately (by this method), or on save,
          if a lock prevents the setting of the property. Implementations may
          differ on when this validation is performed.
          
        o Raise RepositoryException if another error occurs.
        """

    def update(srcWorkspaceName):
        """ Update the node from its correspondent in another workspace.
        
        o If this node has a corresponding node in the workspace
          'srcWorkspaceName', then replace this node and its subtree with
          a clone of the corresponding node and its subtree.
          
        o If this node does not have a corresponding node in the workspace
          'srcWorkspaceName', then the update method has no effect.
          
        o If the update succeeds, the changes made to this node and its
          subtree are persisted immediately, there is no need to call save.
          
        o Note that update does not respect the checked-in status of nodes.
          An update may change a node even if it is currently checked-in
          (this fact is only relevant in an implementation that supports
          versioning).
          
        o Raise NoSuchWorkspaceException if the specified 'srcWorkspace'
          does not exist
          
        o Raise AccessDeniedException if the current session does not have
          sufficient permissions to perform the operation.

        o Raise InvalidItemStateException if this Session (not necessarily
          this Node) has pending unsaved changes.
          
        o Raise LockException if a lock prevents the update
        
        o Raise RepositoryException if another error occurs.
        """

    def getCorrespondingNodePath(workspaceName):
        """ Return the absolute path of the node in the specified
           workspace that corresponds to this node.
           
        o Raise ItemNotFoundException if no corresponding node exists.
        
        o Raise NoSuchWorkspaceException if the specified workspace does
          not exist.
          
        o Raise AccessDeniedException if the current Session does not have
          sufficient permissions to perform this operation.
          
        o Raise RepositoryException if another error occurs.
        """

    def addMixin(mixinName):
        """ Add the specified mixin node type to this node.
        
        o Also adds 'mixinName' to this node's 'jcr:mixinTypes' property
          immediately.
          
        o Semantically, the mixin node type assignment may take effect
          immediately and at the very least, it must take effect on save.
          
        o Raise ConstraintViolationException either immediately or on save
          if a conflict with another assigned mixin or the primary node type
          or for an implementation-specific reason. Implementations may differ
          on when this validation is done.
          
        o In some implementations it may only be possible to add mixin types
          before a node is first saved, and not after. In such cases any later
          calls to addMixin will throw a ConstraintViolationException either
          immediately or on save.
          
        o Raise NoSuchNodeTypeException if the specified mixin node type is
          not recognized, either immediately or on save. Implementations may
          differ on when this validation is done.
          
        o Raise VersionException either immediately or on save if this node
          is versionable and checked-in, or is non-versionable but its
          nearest versionable ancestor is checked-in.  Implementations may
          differ on when this validation is done.
          
        o Raise LockException either immediately or on save if a lock
          prevents the addition of the mixin. Implementations may differ
          on when this validation is done.
          
        o Raise RepositoryException if another error occurs.
        """

    def removeMixin(mixinName):
        """ Remove the specified mixin node type from this node.
        
        o Also removes mixinName from this node's 'jcr:mixinTypes' property
          immediately.
          
        o Semantically, the mixin node type removal may take effect
          immediately and at the very least, it must take effect on save.
          
        o Raise NoSuchNodeTypeException if this node does not have the
          specified mixin, either immediately or on save. Implementations
          may differ on when this validation is done.
          
        o Raise ConstraintViolationException either immediately or on save
          if the removal of a mixin is not allowed. Implementations are free
          to enforce any policy they like with regard to mixin removal and may
          differ on when this validation is done.
          
        o Raise VersionException either immediately or on save if this node
          is versionable and checked-in or is non-versionable but its nearest
          versionable ancestor is checked-in. Implementations may differ on
          when this validation is done.

        o Raise LockException either immediately or on save if a lock
          prevents the removal of the mixin. Implementations may differ on
          when this validation is done.
          
        o Raise RepositoryException thrown if another error occurs.
        """

    def canAddMixin(mixinName):
        """ Return True if the specified mixin node type can be added to
            this node;  else False.
            
        o A result of False must be returned in each of the following cases:
        
          - The mixin's definition conflicts with an existing primary or
            mixin node type of this node. 

          - This node is versionable and checked-in or is nonversionable
            and its nearest versionable ancestor is checked-in.

          - This node is protected (as defined in this node's NodeDefinition,
            found in the node type of this node's parent). 
             
          - An access control restriction would prevent the addition of
            the mixin. 
            
          - A lock would prevent the addition of the mixin. 
          
          - An implementation-specific restriction would prevent the addition
            of the mixin.
            
        o Raise NoSuchNodeTypeException if the specified mixin node type name
          is not recognized.
          
        o Raise RepositoryException thrown if another error occurs.
        """
