""" JSR-170 Session interface, Level 2

$Id: session.py 514 2005-07-11 17:57:16Z tseaver $
"""
from jsr170.level1 import ISession as ISessionLevel1

class ISession(ISessionLevel1):
    """ Extend ISessionLevel1 to provide write access.
    """

    def save():
        """ Validate and save all pending changes currently recorded in this
            session.
        
        o If validation of all pending changes succeeds, then this change
          information is cleared from the session.  If the 'save' occurs
          outside a transaction, the changes are persisted and thus made
          visible to other 'Sessions'.  If the 'save' occurs within a
          transaction, the changes are not persisted until the transaction
          is committed.

        o If validation fails, then no pending changes are saved and they
          remain recorded on the session.  There is no best-effort or
          partial save.

        o When an item is saved the item in persistent storage to which
          pending changes are written is determined as follows:

          - If the transient item has a UUID, then the changes are written
            to the persistent item with the same UUID.

          - If the transient item does not have a UUID then its nearest
            ancestor with a UUID, or the root node (whichever occurs first)
            is found, and the relative path from the node in persistent node
            with that UUID is used to determine the item in persistent storage
            to which the changes are to be written.

        o Note that 'save' uses the same rules to match items between
          transient storage and persistent storage 'INode.update' does to
          match nodes between two workspaces.
     
        o As a result of these rules, a 'save' of an item that has a UUID will
          succeed even if that item has, in the meantime, been moved in
          persistent storage to a new location (that is, its path has changed).

        o Raise InvalidItemStateException on 'save' of a non-UUID item 
          if it has, in the meantime, been moved in persistent storage to
          a new location.
          
        o Raise InvalidItemStateException on 'save' of a non-UUID item if
          it has, in addition to being moved, been replaced in its original
          position by a UUID-bearing item.

        o Raise AccessDeniedException if any of the changes to
          be persisted would violate the access privileges of this session.

        o Raise ReferentialIntegrityException if any of the changes to be
          persisted would cause the removal of a node that is currently the
          target of a 'REFERENCE' property, provided that this session has
          read access to that 'REFERENCE' property.  If, on the other hand,
          this session does not have read access to the 'REFERENCE' property
          in question, Raise AccessDeniedException instead.

        o Raise ItemExistsException' if any of the changes to be persisted
          would be prevented by the presence of an already existing item in
          the workspace.

        o Raise ConstraintViolationException if any of the changes to be
          persisted would violate a node type restriction.  Additionally, a
          repository may use this exception to enforce implementation- or
          configuration-dependant restrictions.

        o Raise LockException if any of the changes to be persisted would
          violate a lock.

        o Raise InvalidItemStateException if any of the changes to be
          persisted conflicts with a change already persisted through another
          session and the implementation is such that this conflict can only
          be detected at save-time and therefore was not detected earlier,
          at change-time.

        o Raise VersionException if the 'save' would make a result in
          a change to persistent storage that would violate the read-only
          status of a checked-in node.

        o Raise NoSuchNodeTypeException is thrown if the 'save' would result
          in the addition of a node with an unrecognized node type.

        o Raise RepositoryException will be thrown if another error occurs.
        """

    def refresh(keepChanges):
        """ Synchronize the session with the persistent storage.

        o If 'keepChanges' is False, discard all pending changes currently
          recorded in this session and return all items to reflect the
          current saved state. Outside a transaction this state is simply
          the current state of persistent storage.  Within a transaction,
          this state will reflect persistent storage as modified by changes
          that have been saved but not yet committed.

        o If 'keepChanges' is True, then do not discard pending changes;
          refresh the state of items that do not have changes pending to
          reflect the current saved state, thus revealin changes made by
          other sessions.

        o Raise RepositoryException if an error occurs.
        """

    def hasPendingChanges():
        """ Return True if this session holds pending (i.e, unsaved) changes;
            otherwise returns False.

        o Raise RepositoryException if an error occurs.
        """

    def getValueFactory():
        """ Return an IValueFactory used to create IValue objects
            for use when setting repository properties.

        o Raise UnsupportedRepositoryOperationException if writing to
          the repository is not supported (because this is a level 1-only
          implementation, for example)

        o Raise RepositoryException if another error occurs.
        """

    def move(srcAbsPath, destAbsPath):
        """ Move the node at 'srcAbsPath' (and its entire subtree) to
            the new location at 'destAbsPath'.

        o In order to persist the change, a 'save' must be called on either
          the session or a common ancestor to both the source and destination
          locations.

          Note that this behaviour differs from that of 'IWorkspace.move',
          which operates directly in the persistent workspace and does not
          require a 'save'.

        o This method cannot be used to move just an individual property by
          itself.  It moves an entire node and its subtree (including, of
          course, any properties contained therein).

        o Raise ConstraintViolationException either immediately or on 'save'
          if performing this operation would violate a node type or
          implementation-specific constraint.  Implementations may differ on
          when this validation is performed.

        o Raise ConstraintViolationException on 'save' if an attempt is
          made to seperately 'save' either the source or destination node.

        o Raise RepositoryException if the provided 'destAbsPath' has an
          index on its final element.  Strictly speaking, the 'destAbsPath'
          parameter is actually an *absolute path* to the parent node of the
          new location, appended with the new *name* desired for the moved
          node. It does not specify a position within the child node ordering
          (if such ordering is supported).  If ordering is supported by the
          node type of the parent node of the new location, then the newly
          moved node is appended to the end of the child node list.

        o Raise PathNotFoundException if no node exists at 'srcAbsPath' or
          no node exists one level above 'destAbsPath' (in other words,
          there is no node that will serve as the parent of the moved item),
          either immediately or on 'save'. Implementations may differ
          on when this validation is performed.

        o Raise ItemExistsException either immediately or on 'save' if a
          property already exists at 'destAbsPath' or a node already exists
          there and same-name siblings are not allowed. Implementations may
          differ on when this validation is performed.

        o Raise VersionException is thrown either immediately or on 'save'
          if the parent node of 'destAbsPath' or the parent node of
          'srcAbsPath' is versionable and checked-in, or is non-versionable
          and its nearest versionable ancestor is checked-in.  Implementations
          may differ on when this validation is performed.

        o Raise LockException either immediately or on 'save' if a lock
          prevents the 'move'. Implementations may differ on when this
          validation is performed.
        """

    def getImportContentHandler(parentAbsPath, uuidBehavior):
        """ Return an 'xml.sax.handler.ContentHandler' which can be used to
            push SAX events into the repository.

        o 'parentAbsPath' is the absolute path of a node under which
          (as child) the imported subtree will be built.

        o 'uuidBehavior' is a flag that governs how incoming UUIDs
          are handled.  It must be one of the four values specified in the
          IImportUUIDBehavior enumeration.

        o If the incoming XML stream (in the form of SAX events) does not
          appear to be a JCR *system view* XML document then it is
          interpreted as a JCR *document view* XML document.

        o The incoming XML is deserialized into a subtree of items
          immediately below the node at 'parentAbsPath'.

        o This method simply returns the 'ContentHandler' without altering
          the state of the session;  the actual deserialization to the
          session transient space is done through the methods of the
          'ContentHandler'. Invalid XML data will cause the 'ContentHandler'
          to throw a 'SAXException'.

        o As SAX events are fed into the 'ContentHandler', the tree of
          new items is built in the transient storage of the session.
          In order to persist the new content, 'save' must be called.
          The advantage of this through-the-session method is that
          (depending on which constraint checks the implementation leaves
          until 'save') structures that violate node type constraints can be
          imported, fixed and then saved. The disadvantage is that a large
          import will result in a large cache of pending nodes in the session.
         
          See 'IWorkspace.getImportContentHandler' for a version of this
          method that does not go through the session.

        o The flag 'uuidBehavior' governs how the UUIDs of incoming
          (deserialized) nodes are handled. There are four options:

          'IImportUUIDBehavior.IMPORT_UUID_CREATE_NEW' --
            Incoming referenceable nodes are added in the same way that new
            node is added with 'Node.addNode'. That is, they are either
            assigned newly created UUIDs upon addition or upon 'save'
            (depending on the implementation). In either case, UUID collisions
            will not occur.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_REMOVE_EXISTING' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace then the already existing node
            (and its subtree) is removed from wherever it may be in the
            workspace before the incoming node is added. Note that this can
            result in nodes *disappearing* from locations in the workspace
            that are remote from the location to which the incoming subtree
            is being written. Both the removal and the new addition
            will be persisted on 'save'.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_REPLACE_EXISTING' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace, then the already-existing node
            is replaced by the incoming node in the same position as the
            existing node. Note that this may result in the incoming subtree
            being disaggregated and 'spread around' to different locations
            in the workspace. In the most extreme case this behavior may
            result in no node at all being added as child of 'parentAbsPath'.
            This will occur if the topmost element of the incoming XML has the
            same UUID as an existing node elsewhere in the workspace.
            The change will be persisted on 'save'.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace then a 'SAXException' is thrown
            by the 'ContentHandler' during deserialization.

        o Unlike 'Workspace.getImportContentHandler', this method does not
          necessarily enforce all node type constraints during deserialization.
          Those that would be immediately enforced in a normal write method
          ('Node.addNode', 'Node.setProperty' etc.) of this implementation
          cause the returned 'ContentHandler' to throw an immediate
          'SAXException' during deserialization.

        o All other constraints are checked on save, just as they are in
          normal write operations. However, which node type constraints
          are enforced depends upon whether node type information in the
          imported data is respected, and this is an implementation-specific
          issue.

        o The 'ContentHandler' raises 'SAXException' during deserialization
          if 'uuidBehavior' is set to 'IMPORT_UUID_COLLISION_REMOVE_EXISTING'
          and an incoming node has the same UUID as the node at
          'parentAbsPath' or one of its ancestors.

        o Raise PathNotFoundException is thrown either immediately or on
          'save' if no node exists at 'parentAbsPath'. Implementations may
          differ on when this validation is performed

        o Raise ConstraintViolationException either immediately or on 'save'
          if the new subtree cannot be added to the node at 'parentAbsPath'
          due to node-type or other implementation-specific constraints, and
          this can be determined before the first SAX event is sent.  
          Implementations may differ on when this validation is performed.

        o Raise VersionException either immediately or on 'save' if the node
          at 'parentAbsPath' is versionable and checked-in, or is
          non-versionable but its nearest versionable ancestor is checked-in.
          Implementations may differ on when this validation is performed.

        o Raise LockException either immediately or on 'save' if a lock
          prevents the addition of the subtree. Implementations may differ on
          when this validation is performed.

        o Raise RepositoryException if another error occurs.
        """

    def importXML(parentAbsPath, inputStream, uuidBehavior):
        """ Deserialize an XML document and adds the resulting item
           subtree as a child of the node at parentAbsPath.

        o 'parentAbsPath' is the absolute path of the node below which the
          deserialized subtree is added.

        o 'inputStream' is an file-like object from which the XML to be
          deserilaized is read (it must have a 'read' method).

        o 'uuidBehavior' a four-value flag that governs how incoming UUIDs
           are handled.

        o If the incoming XML stream does not appear to be a JCR *system view*
          XML document then it is interpreted as a *document view*
          XML document.
          
        o The tree of new items is built in the transient storage of the
          session.  In order to persist the new content, 'save' must be called.
          The advantage of this through-the-session method is that
          (depending on which constraint checks the implementation leaves
          until 'save') structures that violate node type constraints can be
          imported, fixed and then saved. The disadvantage is that a large
          import will result in a large cache of pending nodes in the session.
         
          See 'IWorkspace.importXML' for a version of this
          method that does not go through the session.

        o The flag 'uuidBehavior' governs how the UUIDs of incoming
          (deserialized) nodes are handled. There are four options:

          'IImportUUIDBehavior.IMPORT_UUID_CREATE_NEW' --
            Incoming referenceable nodes are added in the same way that new
            node is added with 'Node.addNode'. That is, they are either
            assigned newly created UUIDs upon addition or upon 'save'
            (depending on the implementation). In either case, UUID collisions
            will not occur.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_REMOVE_EXISTING' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace then the already existing node
            (and its subtree) is removed from wherever it may be in the
            workspace before the incoming node is added. Note that this can
            result in nodes *disappearing* from locations in the workspace
            that are remote from the location to which the incoming subtree
            is being written. Both the removal and the new addition
            will be persisted on 'save'.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_REPLACE_EXISTING' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace, then the already-existing node
            is replaced by the incoming node in the same position as the
            existing node. Note that this may result in the incoming subtree
            being disaggregated and 'spread around' to different locations
            in the workspace. In the most extreme case this behavior may
            result in no node at all being added as child of 'parentAbsPath'.
            This will occur if the topmost element of the incoming XML has the
            same UUID as an existing node elsewhere in the workspace.
            The change will be persisted on 'save'.

          'IImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW' --
            If an incoming referenceable node has the same UUID as a node
            already existing in the workspace then raise 'ItemExistsException'.

        o Unlike 'Workspace.getImportContentHandler', this method does not
          necessarily enforce all node type constraints during deserialization.
          Those that would be immediately enforced in a normal write method
          ('Node.addNode', 'Node.setProperty' etc.) of this implementation
          raise an immediate 'ConstraintViolationException' during
          deserialization.

        o All other constraints are checked on save, just as they are in
          normal write operations. However, which node type constraints
          are enforced depends upon whether node type information in the
          imported data is respected, and this is an implementation-specific
          issue.

        o Raise ConstraintViolationException immediatedly
          if 'uuidBehavior' is set to 'IMPORT_UUID_COLLISION_REMOVE_EXISTING'
          and an incoming node has the same UUID as the node at
          'parentAbsPath' or one of its ancestors.

        o Raise PathNotFoundException is thrown either immediately or on
          'save' if no node exists at 'parentAbsPath'. Implementations may
          differ on when this validation is performed

        o Raise ConstraintViolationException either immediately or on 'save'
          if the new subtree cannot be added to the node at 'parentAbsPath'
          due to node-type or other implementation-specific constraints, and
          this can be determined before the first SAX event is sent.  
          Implementations may differ on when this validation is performed.

        o Raise VersionException either immediately or on 'save' if the node
          at 'parentAbsPath' is versionable and checked-in, or is
          non-versionable but its nearest versionable ancestor is checked-in.
          Implementations may differ on when this validation is performed.

        o Raise LockException either immediately or on 'save' if a lock
          prevents the addition of the subtree. Implementations may differ on
          when this validation is performed.

        o Raise RepositoryException if another error occurs.
        """

class ISessionWithLocks(ISession):
    """ Extend ISession to allow optional locking.
    """
    def addLockToken(token):
        """ Adds the specified lock token to this session.

        o 'token' is a lock token (a string) representing ownership of
          the lock.
        
        o Holding a lock token allows the session object of the lock owner
          to alter nodes that are locked by the lock specified by that
          particular lock token.
        """

    def getLockTokens():
        """ Return a sequence of lock tokens currently held by this session.

        o Returned sequence elements are strings.
        """

    def removeLockTocken(token):
        """ Removes the specified lock token from this session.

        o 'token' is a lock token (a string) representing ownership of
          the lock.
        """
